Require Coq.omega.Omega.

(* ----------------------------------------------------------------------------------------------------------------- *)

(* Arithmetic lemmas on the natural numbers. *)

Lemma arith_plus_commutative_23:
  forall i j k, i + j + k = i + k + j.
Proof.
  intros. omega.
Qed.

Lemma arith_plus_associative:
  forall i j k,
  i + (j + k) = i + j + k.
Proof.
  intros. omega.
Qed.

Lemma arith_plus_zero:
  forall n, n + 0 = n.
Proof.
  intros. omega.
Qed.

Lemma arith_minus_zero:
  forall n, n - 0 = n.
Proof.
  intros. omega.
Qed.

Lemma arith_plus_successor:
  forall m n,
  m + S n = S (m + n).
Proof.
  intros. omega.
Qed.

Lemma arith_minus_successor:
  forall m n,
  m - S n = (m - 1) - n.
Proof.
  intros. omega.
Qed.

Lemma arith_cancellation:
  forall m n,
  n >= m ->
  m + (n - m) = n.
Proof.
  intros. omega.
Qed.

Lemma arith_successor_minus_one:
  forall i,
  S i - 1 = i.
Proof.
  intros. omega.
Qed.

Lemma arith_minus_plus:
  forall m n p,
  m - (n + p) = m - p - n.
Proof.
  intros. omega.
Qed.

(* ----------------------------------------------------------------------------------------------------------------- *)

(* Arithmetic lemmas on the relative numbers. *)

Require Import Coq.ZArith.ZArith.
Local Open Scope Z_scope.

Lemma Z_nat_Z_abs:
  forall z,
  z >= 0 ->
  Z_of_nat (Zabs_nat z) = z.
Proof.
  intros. rewrite inj_Zabs_nat. apply Zabs_eq. omega.
Qed.

Lemma Z_nat_Z_abs_either_way:
  forall z,
  Z_of_nat (Zabs_nat z) >= z.
Proof.
  intro. rewrite inj_Zabs_nat.
  apply Zabs_ind; omega.
Qed.

